#!/usr/bin/env python

"""
JBoss, Home of Professional Open Source Copyright 2009, Red Hat
Middleware LLC, and others contributors as indicated by the @authors
tag. All rights reserved. See the copyright.txt in the distribution
for a full listing of individual contributors. This copyrighted
material is made available to anyone wishing to use, modify, copy, or
redistribute it subject to the terms and conditions of the GNU Lesser
General Public License, v. 2.1. This program is distributed in the
hope that it will be useful, but WITHOUT A WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU Lesser General Public License for more details.
You should have received a copy of the GNU Lesser General Public
License, v.2.1 along with this distribution; if not, write to the Free
Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
"""

import cPickle
import sys
import simplejson as json
import qp
import urlparse

class TxHandler():
        def __init__(self, path):
                query = path.strip('/').split('?')
                tokens = query[0].split('/')
#                print 'striped tok0=' + tokens[0]
                self.code = 400
                self.service = ''
                self.status = 'active'
                self.method = 'status'
                self.methods = { 'prepare' : self.do_prepare,
                                 'commit' : self.do_commit,
                                 'rollback' : self.do_rollback,
                                 'forget' : self.do_forget,
                                 'status' : self.do_status}

                #tokens = path.strip('/').split('/')

                if len(tokens) > 2 and tokens[0] == 'participant':
                        self.service = tokens[1]
                        self.txid = tokens[2]
                        if len(tokens) > 3:
                                self.method = tokens[3]

		print self.service, ' txId: ', self.txid, ' method: ', self.method

        def process(self):
                print 'process: method=' + self.method + ' txid=' + self.txid
                if self.method in self.methods:
                        self.code = 201
                        self.methods[self.method](self.txid)

        def do_commit(self, txid):
                print 'do_commit'
                self.status = 'commit_ok'

        def do_rollback(self, txid):
                print 'do_rollback'
                self.status = 'rollback_ok'

        def do_prepare(self, txid):
                print 'do_prepare'
                self.status = 'prepare_ok'

        def do_forget(self, txid):
                print 'do_forget'
                self.status = 'forget_ok'

        def do_status(self, txid):
                print 'do_status'
                self.status = 'status_ok'

class Work:
   'Common base class for all work'
   count = 0

   def __init__(self, name, status):
      self.props = {}
      self.props['name'] = name
      self.props['status'] = status
      self.name = name
      self.status = status
      Work.count += 1
   
   def update(self, updates):
      self.props.update(updates)

   def get(self):
      return self.props

   def displayCount(self):
     print "Total Work %d" % Work.count

   def displayWork(self):
      print self.props
      #print "Name : ", self.name,  ", Status: ", self.status

   def __str__( self ) :
      return "(" + str( self.name ) + ", " + \
                   str( self.status ) + ")"

#work = {'Taxi': Work('Taxi', 'New'), 'Theatre': Work('Theatre', 'New'), 'Restaurant': Work('Restaurant', 'New')};

def start_branch(service, txId, port):
	tokens = txId.split('/')
	tx = tokens[len(tokens) - 1]
	purl = 'http://localhost:' + str(port) + '/participant/' + service + '/'+ tx
	print 'enlisting url ', purl
	try:
		f = open('tmp/' + tx, 'w')
		f.write(purl)
		f.close()
		qp.put('tmp/' + tx, txId)
		return purl
	except IOError, arg:
		return None
	except:
		return None


def get_work(service, tx):
	fn = 'tmp/work.' + tx
	try:
		FILE = open(fn, 'r')
		work = cPickle.load(FILE)
		FILE.close()
	except IOError, arg:
		print 'file open error ', arg

	return work

def post_handler(self, port):
	print 'POST request path=' + self.path
	res = TxHandler(self.path)
	work = get_work(res.service, res.txid)
	if work is None:
		print 'POST ERROR: no such transactional work'
		return

	res.process()
	status = {}
	status['status'] = res.status
	service = work[res.service]
	service.update(status)

#	print 'POST tx status: ', res.status, ' props: ', service.props
	FILE = open('tmp/work.' + res.txid, 'w')
	cPickle.dump(work, FILE)
	FILE.close()

	self.send_response(res.code)
	self.send_header('location', 'http://localhost:' + str(port) + '/participant/' + res.service + '/' + res.txid + '/' + res.status)
	self.send_header('Content-Type', 'text/javascript')
	self.send_header('Content-Length', 0)

	self.end_headers()


def get_handler(self, port):
	path = urlparse.urlparse(self.path)
	args = dict([param.split('=') for param in path.query.split('&')])

	if 'txId' not in args:
		return []
	txId = args['txId']
	tokens = txId.split('/')
	tx = tokens[len(tokens) - 1]
	fn = 'tmp/work.' + tx

	if 'service' not in args:
		print 'Missing service name'
		return []

	try:
		FILE = open(fn, 'r')
		work = cPickle.load(FILE)
		FILE.close()
	except IOError, arg:
		print 'file open error ', arg, ' args: ', args
		work = {'Taxi': Work('Taxi', 'New'), 'Theatre': Work('Theatre', 'New'), 'Restaurant': Work('Restaurant', 'New')};

	if args['service'] not in work:
		print 'Unsupported service ', args['service']
		return []

	service = work[args['service']]
#	print 'updating ', service.get(), ' args ', args
	service.update(args)

	if not 'url' in service.get():
		url = start_branch(args['service'], txId, port)
		if url is None:
			return json.dumps({'status':'Invalid id'})
		service.update({'url':url})

	if len(service.props) > 2 and not 'status' in service.props:
		service.update({'status':'Modified'})
	FILE = open('tmp/work.' + tx, 'w')
	cPickle.dump(work, FILE)
	FILE.close()
#	print 'persisted ', json.dumps(service.get())

	resp = '['
	resp += ''.join(json.dumps(service.get()))
	resp += ']'

	if 'callback' in args:
		resp = ('%s(%s);' % (args['callback'], resp))

	self.send_response(200)
	self.send_header('Content-Type', 'text/javascript')
	self.send_header('Content-Length', len(resp))
	self.send_header('Expires', '-1')
	self.send_header('Cache-Control', 'no-cache')
	self.send_header('Pragma', 'no-cache')
	self.end_headers()

	self.wfile.write(resp)
	self.wfile.flush()

def test1(args):
	mods1 = {}
	for arg in args:
		print arg
		nvp = arg.split('=')
		mods1[nvp[0]] = nvp[1];
#	print mods1
	mods1 = get_handler(mods1, 8001)
	print mods1

def test2():
	FILE = open("work.txt", 'w')
	service = work['Taxi']
	service.update({'required':'yes'})
	cPickle.dump(work, FILE)
	FILE.close()

	print 'reading back ', service.get()
	FILE = open("work.txt", 'r')
	work = cPickle.load(FILE)
	service = work['Taxi']
	FILE.close()
	print 'read back ', service.get()

if __name__ == "__main__":
#	clear; python tx.py service=Taxi txId=tx01 type=seat
	sys.argv.pop(0)
	test1(sys.argv)
