/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2009
 * @author JBoss Inc.
 */
package org.jboss.jbossts.rts.demo.client;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ListBox;

import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.KeyboardListener;

import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.TabPanel;
import com.google.gwt.user.client.ui.RootPanel;

import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.ServiceDefTarget;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;

import com.google.gwt.user.client.rpc.AsyncCallback;

//JRE Emulation
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

//Demo classes
import org.jboss.jbossts.rts.demo.client.service.Restaurant;
import org.jboss.jbossts.rts.demo.client.service.StatusPanel;
import org.jboss.jbossts.rts.demo.client.service.Taxi;
import org.jboss.jbossts.rts.demo.client.service.Theatre;
import org.jboss.jbossts.rts.demo.client.service.ServicePanel;

// based on a GWT tutorial:
// http://code.google.com/webtoolkit/doc/1.6/DevGuideServerCommunication.html#DevGuideRPCDeployment
// http://code.google.com/webtoolkit/tutorials/1.5/JSON.html

public class RESTDemoApp implements EntryPoint
{
	private static final String THEATRE_URL = "http://localhost:8000/";
	private static final String RESTAURANT_URL = "http://localhost:8001/";
	private static final String TAXI_URL = "http://localhost:8002/";

	private static final String[] faultNames = {"none", "abort_halt", "abort_suspend", "prepare_halt", "prepare_suspend",
		"commit_halt", "commit_suspend", "h_commit", "h_rollback", "h_hazard", "h_mixed"};

	private StatusPanel statusPanel = new StatusPanel(0, "", 0, false);
	private Panel controlPanel;
	private Panel bookingPanel;
	private ListBox faults;
	private TextBox txTb;

	private ServicePanel[] services = {
		new Theatre(THEATRE_URL, statusPanel),
		new Restaurant(RESTAURANT_URL, statusPanel),
		new Taxi(TAXI_URL, statusPanel)
	};

	/**
	 * Entry point method.
	 */
	public void onModuleLoad()
	{
		Panel app = new VerticalPanel();

		app.add(getStatusPanel());
		app.add(getControlPanel());
		app.add(getBookingPanel());

		RootPanel.get("app-frame").add(app);
	}

	private Panel getStatusPanel()
	{
		return statusPanel;
	}

	private Panel getControlPanel()
	{
		final Button startButton = new Button("Book a Trip");
		final Button checkButton = new Button("Trip Status");

		txTb = new TextBox();
		controlPanel = new HorizontalPanel();
		controlPanel.setVisible(true);
		controlPanel.add(startButton);
		controlPanel.add(checkButton);
		controlPanel.add(txTb);

		startButton.addClickListener(new ClickListener() {
			public void onClick(Widget event) {
				controlPanel.setVisible(false);
				bookingPanel.setVisible(true);

				for (ServicePanel service : services)
					service.reset();

				faults.setSelectedIndex(0);
				statusPanel.clearStatus("Starting");
				startTx();
			}
		});

		checkButton.addClickListener(new ClickListener() {
			public void onClick(Widget event) {
					showTx(services, txTb.getText());
			}
		});

		txTb.addKeyboardListener(new KeyboardListener() {
			public void onKeyDown(Widget widget, char c, int i) {}
			public void onKeyUp(Widget widget, char c, int i) { }
			public void onKeyPress(Widget sender, char keyCode, int modifiers) {
				if (keyCode == KeyboardListener.KEY_ENTER) {
					showTx(services, txTb.getText());
				}
			}
		});

		return controlPanel;
	}

	private Panel getBookingPanel()
	{
		final Button confirmButton = new Button("Confirm Booking");
		final Button abortButton = new Button("Cancel Booking");
		final HorizontalPanel faultPanel = new HorizontalPanel();
		HorizontalPanel panel = new HorizontalPanel();
		TabPanel tabPanel =  new TabPanel();

		bookingPanel =  new VerticalPanel();
		tabPanel.setWidth("500px");
		bookingPanel.setVisible(false);

		for (ServicePanel service : services)
			tabPanel.add(service, service.getName());

		confirmButton.addStyleDependentName("tx");
		panel.add(confirmButton);
		abortButton.addStyleDependentName("tx");
		panel.add(abortButton);

		faults = ServicePanel.createListBox(true, "dropdown", 0, faultNames);
		faults.addStyleDependentName("tx");
		faults.setSelectedIndex(0);
		panel.add(faults);
		faultPanel.add(new Label("Add Fault"));
		faultPanel.add(faults);
		panel.add(faultPanel);

		bookingPanel.add(panel);
		bookingPanel.add(tabPanel);

		confirmButton.addClickListener(new ClickListener() {
			public void onClick(Widget event) {
				controlPanel.setVisible(true);
				bookingPanel.setVisible(false);
				statusPanel.setStatus("Confirming");
				endTx(true);
			}
		});

		abortButton.addClickListener(new ClickListener() {
			public void onClick(Widget event) {
				controlPanel.setVisible(true);
				bookingPanel.setVisible(false);
				statusPanel.setStatus("Cancelling");
				endTx(false);
			}
		});

		return bookingPanel;
	}

	private static TxServiceAsync app = null;
	private TxServiceAsync getServiceInstance()
	{
		if (app == null) {
			app = (TxServiceAsync) GWT.create(TxService.class);
			((ServiceDefTarget) app).setServiceEntryPoint(GWT.getModuleBaseURL() + "TxDemo");
		}

		return app;
	}

	private void showTx(ServicePanel[] services, String txId)
	{
		statusPanel.setTxId(txId);
		for (ServicePanel service : services)
			service.refresh();
	}

	// start a transaction via RPC
	private void startTx() {
		// Create a callback object to retrieve the response
		AsyncCallback<String> callback = new AsyncCallback<String>() {
			public void onFailure(Throwable e) {
				onSuccess("Error: " + e.getMessage());
				//String msg = "Start booking error: " + e.getMessage();
				//statusPanel.clearStatus(msg);
			}

			public void onSuccess(String txid) {
				if (txid != null && txid.startsWith("Error")) {
					statusPanel.clearStatus(txid);
					controlPanel.setVisible(true);
					bookingPanel.setVisible(false);
				} else {
					statusPanel.setStatus("In progress");
					statusPanel.setTxId(txid);
					txTb.setText(statusPanel.getId());
				}
			}
		};

		txTb.setText("");
		getServiceInstance().startTx("clientId", callback);
	}

	// end a transaction via RPC
	private void endTx(final boolean commit) {
		AsyncCallback<String> callback = new AsyncCallback<String>() {
			public void onFailure(Throwable e) {
				String msg = (commit ? "Confirm" : "Cancel") + " booking error: " + e.getMessage();
				statusPanel.setStatus(msg);
			}

			public void onSuccess(String result) {
				String msg = (commit ? "Confirmed" : "Canceled");
				statusPanel.setStatus(msg);
				txTb.setText(statusPanel.getId());
				showTx(services, txTb.getText());
			}
		};

		String fault = faults.getItemText(faults.getSelectedIndex());

		if (commit)
			getServiceInstance().endTx(statusPanel.getTxId(), "fault=" + fault, callback);
		else
			getServiceInstance().rollbackTx(statusPanel.getTxId(), "fault=" + fault, callback);

		startTimer();
	}

	private Timer startTimer()
	{
		Timer statusTimer = new Timer() {
			@Override
			public void run() {
				if (statusPanel.is_complete())
					cancel();
				else
					for (ServicePanel service : services)
						if (statusPanel.is_in_progress(service.getName()))
							service.refresh();
			}
		};

		statusTimer.scheduleRepeating(10000);

		return statusTimer;
	}
}
