/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2009
 * @author JBoss Inc.
 */
package org.jboss.jbossts.rts.demo.server;

import org.jboss.resteasy.util.HttpResponseCodes;
import org.jboss.jbossts.rts.util.TxUtil;
import org.jboss.jbossts.rts.provider.HttpResponseException;

import java.io.IOException;

/**
 * TODO
 */
public class TxHelper
{
    private static String[] HOSTS = {"localhost"};
    private static int[] PORTS = {9099}; //TxUtil.getIntProperty("PORT", "9096");

    static
    {
        //TxUtil.setEndpoint("localhost", PORT, "");
        TxUtil.setHost(HOSTS[0]);
        TxUtil.setPort(PORTS[0]);

    }

    public static void assertTrue(String message, boolean ok)
    {
        if (!ok)
            System.err.println(message);
    }

    public static String startTx(long timeout, boolean fail) throws IOException
    {
        String tx = null;

        try {
            String query = "clientId=12345" + (timeout == 0 ? "" : "&timeout=" + timeout);

            tx = TxUtil.doPost(new int[] {HttpResponseCodes.SC_CREATED}, TxUtil.txURI + "/begin?" + query);
//            log.trace("Started tx: " + tx);
            if (fail)
                assertTrue("start tx should have failed", false);
        } catch (HttpResponseException e) {
            if (!fail)
                assertTrue(e.getMessage(), false);
			tx = "Error: No available coordinator";
        }

        return tx;
    }

    public static String modifyResource(String tx, String pid, String name, String value)
    {
        try {
            return TxUtil.doPost(new int[] {HttpResponseCodes.SC_OK}, TxUtil.getBaseURI() + "participant/work/" + pid,
                    "name=" + name, "value=" + value, "context=" + tx);
        } catch (HttpResponseException e) {
            assertTrue(e.getMessage(), false);
            return null;
        }
    }

    public static String getResource(String tx, String pid, String name)
    {
        try {
            return TxUtil.doGet(new int[] {HttpResponseCodes.SC_OK}, TxUtil.getBaseURI() + "participant/work/" + pid,
                    "name=" + name, "context=" + tx);
        } catch (HttpResponseException e) {
            assertTrue(e.getMessage(), false);
            return null;
        }
    }

    public static void terminateTx(String tx, String how, boolean fail, String ... args)
    {
        try {
            TxUtil.doPut(new int[] {HttpResponseCodes.SC_OK}, tx + "/" + how, "", args);
            if (fail)
                assertTrue("commit should not have succeeded", false);
        } catch (HttpResponseException e) {
            if (!fail)
                assertTrue(e.getMessage(), false);
        }
    }

    public static void commitTx(String tx, boolean fail, String ... args)
    {
        terminateTx(tx, "commit", fail, args);
    }

    public static void rollbackTx(String tx, boolean fail, String ... args)
    {
        terminateTx(tx, "rollback", fail, args);
    }
}
