package org.jboss.jbossts.rts.resource;

import com.arjuna.ats.arjuna.coordinator.ActionStatus;
import com.arjuna.ats.arjuna.coordinator.AbstractRecord;
import com.arjuna.ats.arjuna.coordinator.AddOutcome;
import com.arjuna.ats.arjuna.AtomicAction;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlAttribute;

@XmlRootElement(name = "transaction")
public class Transaction extends AtomicAction
{
    private long age = System.currentTimeMillis();
    private String initiator;

    public Transaction()
    {
        super();
    }

    public Transaction(String initiator)
    {
        super();
        this.initiator = initiator;
    }

    @XmlElement
    public String getInitiator()
    {
        return initiator;
    }

    @XmlElement
    public String getAge()
    {
        return Long.toString(System.currentTimeMillis() - age);
    }

    @XmlAttribute
    public String getStatus()
    {
        return ActionStatus.stringForm(super.status());
    }

    public String enlistParticipant(String url)
    {
        RESTRecord p = new RESTRecord(url, get_uid().fileStringForm());
        String coordinatorId = p.get_uid().fileStringForm();
        
        if (add(p) != AddOutcome.AR_REJECTED)
            return coordinatorId;

        return null;
    }

    public void setFault(String fault)
    {
        if (pendingList == null)
            return;

        // only add faults for pending records
        AbstractRecord r = pendingList.peekFront();

        while (r != null)
        {
            if (r instanceof RESTRecord)
                ((RESTRecord) r).setFault(fault);

            r = pendingList.peekNext(r);
        }
    }

    public boolean isTransactionInMidFlight()
    {
        switch ( status() )
        {
            case ActionStatus.RUNNING    :
            case ActionStatus.ABORT_ONLY :
            case ActionStatus.PREPARING  :
            case ActionStatus.COMMITTING :
            case ActionStatus.ABORTING   :
            case ActionStatus.PREPARED   :
                return true;

                //case ActionStatus.INVALID:
            default:
                return false;
        }
//        return !isFinished();
    }

    public boolean isFinished()
    {
        switch ( status() )
        {
            case ActionStatus.COMMITTED  :
            case ActionStatus.H_COMMIT   :
            case ActionStatus.H_MIXED    :
            case ActionStatus.H_HAZARD   :
            case ActionStatus.ABORTED    :
            case ActionStatus.H_ROLLBACK :
                return true;

                //case ActionStatus.INVALID: throw ...
            default:
                return false;
        }
    }

    public boolean isAlive()
    {
        switch ( status() )
        {
            case ActionStatus.RUNNING    :
            case ActionStatus.ABORT_ONLY :
            case ActionStatus.PREPARING  :
            case ActionStatus.COMMITTING :
            case ActionStatus.ABORTING   :
            case ActionStatus.PREPARED   :
                return true;

                //case ActionStatus.INVALID:
            default:
                return false;
        }
    }

    public boolean hasHeuristic()
    {
        switch ( status() )
        {
            case ActionStatus.H_COMMIT   :
            case ActionStatus.H_MIXED    :
            case ActionStatus.H_HAZARD   :
            case ActionStatus.H_ROLLBACK :
                return true;

            default:
                return false;
        }
    }
}
